/******************************************************************************
 King's Quest II SCI Remake
 By Brandon Blume
 ******************************************************************************
 main.sc
 Contains the game's main instance and inventory items.
 ******************************************************************************/
(include "sci.sh")
(include "game.sh")
/******************************************************************************/
(script MAIN_SCRIPT)
/******************************************************************************/
(use "controls")
(use "sound")
(use "syswindow")
(use "cycle")
(use "game")
(use "inv")
(use "user")
(use "menubar")
(use "feature")
(use "obj")
(use "disposeload")
(use "DPath")
/******************************************************************************
 * These are the global variables. You can access them from any script as long
 * as it "use"es this script
 ******************************************************************************/
(local
	gEgo						// 00   /* points to the ego's class */
	gGame						// 01   /* points to the game instance */
	gRoom						// 02   /* points to the current room instance */
	gSpeed 						// 03   /* the game speed (delay each interpreter cycle) */
	gQuitGame = FALSE			// 04   /* if set to TRUE, the game will exit */
	gCred						// 05   /* the credits sequence */
	gIntro						// 06  /* the introduction sequence */
	
	gCast						// 07   /* points to the cast class (list of actors) */
	gRegions					// 08   /* points to the regions class (list of regions) */
	gLocales 					// 09   /* points to the locales class (list of locales) */
	gTimers						// 10   /* points to the timers class (list of timers) */
	gSounds						// 11   /* points to the sounds class (list of sounds) */
	gInv						// 12   /* points to the inventory class */
	gAddToPics					// 13   /* points to the add to pics class */
	gFeatures					// 14   /* points to the add to features class */		
	gSFeatures					// 15   /* points to the add to sfeatures class */
	
	gRoomNumberExit				// 16   /* room number exit */
	gPreviousRoomNumber			// 17   /* the number of the previous room */
	gRoomNumber					// 18   /* the number of the current room */ 
	gDebugOnExit = FALSE		// 19   /* enter debug mode on room exit */ 
	
	gScore = 0					// 20   /* the game score */		
	gMaxScore = 185				// 21   /* the maximum game score */
	gOldScore					// 22   /* previous score */
	
	gCurrentCursor				// 23   /* the number of the current cursor */
	gNormalCursor = 999			// 24   /* the number of the normal cursor (ie. arrow) */
	gLoadingCursor = 997		// 25   /* the number of the loading cursor (ie. hand) */
	
	gDefaultFont = 4			// 26   /* the number of the default font */
	gSaveRestoreFont = 4		// 27   /* the number of the font for the save/restore dialogs*/
	gDeadFont = 0				// 28   /* the number of the font for the dialog when ego dies */
	
	gUserEvent					// 29   /* points to the user's event object */
	gPrintDlg					// 30   /* points to the current Print() window */
	gVolume						// 31   /* the sound volume */
	gVersion					// 32   /* the version string */
	gSaveDirPtr					// 33   /* points to the save directory string */
	gCheckAniWait				// 34   /* the checkAni delay */
	gSetRegions					// 35   /* a flag -- see User:doit() */
	
	gPicAngle					// 36   /* the room's pic angle */
	gOverlay = -1				// 37   /* whether to overlay the picture when drawing */
	gDefaultPicAni				// 38   /* the default pic animation */
	gDefaultPalette				// 39   /* the default palette to use for the pictures (0-3) */
	gCastMotionCue				// 40   /* if set, the cast's motionCue() is called */
	
	gTheWindow					// 41   /* points to the window class */
	gWndColor					// 42   /* the colour of the game's windows foreground (ie. text) */
	gWndBack					// 43   /* the colour of the game's windows background */

	gOldPort					// 44   /* the previous port */
	
	gEgoView = 0				// 45   /* ego's current view number */
	gEgoStoppedView	= 1	     	// 46   /*ego's current stopped view number */
    gEgoWading = 3         		// 47   /*ego wading in water to knees */
    gEgoWadingDeep = 4     	 	// 48   /*ego wading in water to waist */
    gEgoSwimming = 5       	 	// 49   /*ego swimming in water */
    gAnkleWater = 2        	 	// 50   /*water animation for ankle-deep ego*/

	/* hh:mm:ss | gTimeHours:gTimeMinutes:gTimeSeconds */
	/* the time elapsed since the game started */
	gTimeSeconds				// 51   /* the seconds */
	gTimeMinutes				// 52   /* the minutes */
	gTimeHours					// 53   /* the hours */
	
	gCurrentTime				// 54   /* the current time in seconds */
	
	gTheMusic					// 55   /* points to the music class */
	gTheSoundFX					// 56   /* points to the sound fx class */
	gProgramControl 			// 57   /* states whether the program has control or the user */
	
	gGuiView = 901				// 58   The custom GUI View number
	gGuiLoop = 0				// 59   The custom GUI Loop number
	gGuiTopLeft = 0				// 60   The custom GUI topleft Cel
	gGuiTopRight = 1			// 61   The custom GUI topright Cel
	gGuiBottomLeft = 2			// 62   The custom GUI bottomleft Cel
	gGuiBottomRight = 3			// 63   The custom GUI bottomright Cel
	gGuiLeft = 4				// 64   The custom GUI left cel
	gGuiTop = 5					// 65   The custom GUI top cel
	gGuiRight = 6				// 66   The custom GUI right cel
	gGuiBottom = 7				// 67   The custom GUI bottom cel
	gGuiTitleLeft = 8			// 68   The custom GUI left title cel
	gGuiTitleRight = 9			// 69   The custom GUI right title cel
	gGuiTitleTop = 10			// 70   The custom GUI top title cel
	
	gEgoIsSwimming = FALSE 		// 71   Whether Ego is in water or not
	
	gMailBoxClosed = TRUE		// 72   The global variable for Grandma's Mailbox to control whether it is open or losed when you enter the screen
	gBridgeBreak = 0			// 73   Tells how many times you've crossed the bridge since the games started
	gDwarfChestOpen = FALSE		// 74   Whether you've opened the Dwarf's chest or not
	gRidingHoodGoodies = FALSE	// 75   Whether you've given the basket to Red Riding Hood
	gMermaidFlowers = FALSE		// 76   Whether you've given the flowers to the mermaid
	gNeptuneTrident = FALSE     // 77   Whether you've given the trident to King Neptune or not
	gClothBird = FALSE			// 78   Whether you've put the cloth over the birdcage or not
	gAntiqueBird = FALSE		// 79   Whether you've given the nightingale to the Antique Shop owner
	gRubLamp = 0				// 80   How many times you've rubbed the oil lamp
	gCanFlyCarpet = TRUE		// 81   Whether you can ride the carpet or not
	gCarpetFlying = FALSE		// 82	If Ego is currently flying on the magic carpet
	gGrandmaSoup = FALSE		// 83   The global variable for whether you've fed Grandma the chicken soup
	gWearingCloakRing = 0		// 84	Whether you're wearing the cloak, ring or both 
								/*  	0=no,1=cloak,2=ring,3=both */
	gCanLookUnderBed = FALSE	// 85	Whether Grandma has given you permission to look under her bed or not
	gWearingCross = FALSE		// 86   Wearing the cross necklace that the monk gave you or not
	gGhostsKill = TRUE			// 87   If TRUE the Ghosts are visible, if FALSE the ghosts have disappeared
	
    gSugarCube = 0          	// 88   The global variable for whether you've eaten the sugar cube yet or not
                                /*		0=Not eaten yet,1=Eaten,2=Eaten and worn off */
    gLitCandle = FALSE			// 89	Whether you've lit the candle or not
    gChestLocked = TRUE			// 90	Whether the chest in Dracula's tower is locked or not
    gDraculaChestOpen = FALSE	// 91	Whether you've open the chest in Dracula's castle or not
    gDrawerOpen = FALSE			// 92	Whether you've opened the drawer in Dracula's tower or not
    gCoffinOccupied				// 93	Whether the coffin is occupied or not
    gCoffinOpen = FALSE			// 94	Whether the coffin is open or closed
    					
    gDraculaDead = FALSE		// 95	Whether you've killed Dracula or not
    
    gLionState = 0				// 96	Whether the lion is 0=awake, 1=sleeping, or 2=dead
    
    gKeyQuest = 0				// 97	The global variable for what key quest you're on:
	//	0=no quest,1=get 1st key,2=have 1st key,3=get 2nd key,4=have 2nd key,5=get 3rd key,6=have 3rd key
		
)
/******************************************************************************/

/*      New StopWalk Code      */

(class StopWalk of Fwd
   (properties
      client 0
      caller 0
      cycleDir cdFORWARD
      cycleCnt 0
      completed 0
      vWalking 0
      vStopped 0
   )
   (method (init newClient walking stopped)
      (= vWalking walking)
      (= vStopped stopped)
      (super:init(newClient))
      (send client:view(vStopped)) // this avoids the "flicker" when the ego first appears

   )
   (method (doit)
      (if(not (send client:isStopped))
         (send client:view(vWalking))
         (super:doit())
      )
      (else
         (send client:view(vStopped))
      )
   )
)

/*     End of New StopWalk Code    */

(class StopSwim of Fwd
   (properties
      client 0
      caller 0
      cycleDir cdFORWARD
      cycleCnt 0
      completed 0
      vWalking 0
      vStopped 0
   )
   (method (init newClient walking stopped)
      (= vWalking walking)
      (= vStopped stopped)
      (super:init(newClient))
      (send client:view(vStopped)) // this avoids the "flicker" when the ego first appears

   )
   (method (doit)
      (send client:view(vWalking))
         (super:doit())
   )
)

(instance public KQ2 of Game
	(properties)
	(method (init)
        // Set up the base window
		= gTheWindow theWindow
		= gWndColor clBLACK
		= gWndBack clWHITE
		(send gTheWindow:
			color(gWndColor)
		    back(gWndBack)
		)

        // Initialize
		(super:init())

        /******************************
         * Set your game version here *
         ******************************/
		= gVersion "1.0"
		    
		// General initialization stuff
	    = gVolume 15
	    DoSound(sndVOLUME gVolume)
		    
	    (SL:code(statusCode))
	    (TheMenuBar:init())
    
	    (scoreSound:
	    	owner(self)
	    	init()
	    )
	    = gTheMusic theMusic
	    (send gTheMusic:
	    	owner(self)
	    	init()
	    )
		= gTheSoundFX theSoundFX
	    (send gTheSoundFX:
	    	owner(self)
	    	init()
	    )
	    
	    = gEgo ego
	    (User:
	    	alterEgo(gEgo)
	    	blocks(0)
	    	y(150)
	    )
			
		Load(rsFONT		gDeadFont)
		Load(rsFONT		gDefaultFont)
		Load(rsFONT		gSaveRestoreFont)
	
		Load(rsCURSOR	gNormalCursor)
		Load(rsCURSOR	gLoadingCursor)
			
	    (if(HaveMouse())
	    	(send gGame:setCursor(gNormalCursor SET_CURSOR_VISIBLE))
		)(else
	    	(send gGame:setCursor(gNormalCursor SET_CURSOR_VISIBLE 304 174))
		)
			
        /*************************************************
         * Initialize the inventory with it's items here *
         *************************************************/
		(Inv:
			add(
				{Nothing}
				{Sword}				// 01
				{Trident}			// 02
				{Bouquet}			// 03
				{Bracelet}			// 04
				{Stake}				// 05
				{Ruby Ring}			// 06
				{Tiara}				// 07
				{Necklace}			// 08
				{Earrings}			// 09
				{Brooch}			// 10
				{Mallet}			// 11
				{Gold Key}			// 12
				{Empty Basket}		// 13
				{Basket}			// 14
				{Bottle}			// 15
				{Empty Bottle}		// 16
				{Ham}				// 17
				{Chicken Soup}		// 18
				{Black Cloak}		// 19
				{Cross}				// 20
				{Caged Nightingale}	// 21
				{Empty Birdcage}	// 22
				{Oil Lamp}			// 23
				{Cloth}				// 24
				{Pillow}			// 25
				{Fishing Net}		// 26
				{Carpet}			// 27
				{Bridle}			// 28
				{Candle}			// 29
				{Sugar Cube}		// 30
				{Gasping Fish}		// 31
				{Amulet}			// 32
				{Clamshell}			// 33
				{Silver Key}		// 34
			
			)
		)
			
		// Start the room

/**********Restart the game from the title screen********/
//		(self:newRoom(TITLESCREEN_SCRIPT))

/**********Restart the game from the first room**********/
		(if(GameIsRestarting())
	    	(self:newRoom(INITROOMS_SCRIPT))
	    )(else
	    	(self:newRoom(TITLESCREEN_SCRIPT))
		 )
	
	)
	(method (doit)

         (if (send gEgo:isStopped)
            (send gEgo:view(gEgoStoppedView))
         )
         (else
            (send gEgo:view(gEgoView))
         )
      
        (super:doit())
        (if(gProgramControl)
			(User:
				canControl(FALSE)
				canInput(FALSE)
			)
		)
		(if(<> gCurrentTime (= gCurrentTime GetTime(gtTIME_OF_DAY)))
		    (if(>= ++gTimeSeconds 60)
		    	= gTimeSeconds 0
		    	++ gTimeMinutes
		    	(if(>= gTimeMinutes 60)
		    		= gTimeMinutes 0
		    		++ gTimeHours
		    	)
			)
		)
  		(if(> gOldScore gScore)
		    	= gOldScore gScore
				(SL:doit())
		)
		(if(< gOldScore gScore)
		    = gOldScore gScore
			(SL:doit())
		)
	)
	(method (replay)
  		(TheMenuBar:draw())
  		(SL:enable())
  		
  		(if(DoSound(sndSET_SOUND))
		    SetMenu(MENU_TOGGLESOUND #text "Turn Off")
		)(else
		    SetMenu(MENU_TOGGLESOUND #text "Turn On")
		)
		(super:replay())
	)
	(method (newRoom roomNum picAni)
  			DisposePrintDlg()
  			Load(rsFONT		gDeadFont)
  			Load(rsFONT		gDefaultFont)
  			Load(rsFONT		gSaveRestoreFont)

  			Load(rsCURSOR	gNormalCursor)
  			Load(rsCURSOR	gLoadingCursor)
     
			(super:newRoom(roomNum))
			(if(< paramTotal 2)
				= gDefaultPicAni Random(0 5)
			)(else
				= gDefaultPicAni picAni
			)
	)
	(method (startRoom roomNum)
		DisposeLoad(
			NULL
			FILEIO_SCRIPT JUMP_SCRIPT EXTRA_SCRIPT WINDOW_SCRIPT
		        TIMER_SCRIPT FOLLOW_SCRIPT REV_SCRIPT DCICON_SCRIPT
		        DOOR_SCRIPT AUTODOOR_SCRIPT
		)
		DisposeScript(DISPOSELOAD_SCRIPT)
		(if(gDebugOnExit)
			= gDebugOnExit FALSE
			SetDebug()
		)
		(send gTheSoundFX:
			stop()
			number(1)
		)
		(super:startRoom(roomNum))
		(if(== gTheSoundFX 11)
		 	(send gEgo:baseSetter(NormalBase))
		)
	)
	(method (changeScore addScore)
		= gScore + gScore addScore
		(if(> addScore 0)
			(scoreSound:playMaybe())
		)
	)
	(method (changeScoreSilent addScore)
		= gScore + gScore addScore
	)
 (method (wordFail theString)
  (switch(Random(0 4))
/*    (case 0
      FormatPrint("Oh, yeah? Well, \"%s\" this!" theString)
    )
    (case 1
      FormatPrint("You may know the word \"%s\" but it's beyond my vocabulary!" theString)
    )
    (case 2
      FormatPrint("Oh, yeah? Well, I've got your \"%s\" right here!" theString)
    )
    (case 3
      FormatPrint("You don't need to type the word \"%s\" to complete this game!" theString)
    )
*/    (default
      FormatPrint("King Graham, please try to find another word for \"%s\"." theString)
    )
  )
)

  	(method (handleEvent pEvent)
		(var dyingScript, aWord)
		
		//////////////////////////////////////////////////////////////////////
		// ** This is debug functionality                                   //
		// ** Comment it out if you don't want people to cheat in your game //
            //////////////////////////////////////////////////////////////////////
	      (if (== evKEYBOARD (send pEvent:type))
	         // Check for ALT keys
	         (switch (send pEvent:message)
	            (case $2f00 // alt-v
	               Show(1) // Show visual
	            )
	            (case $2e00 // alt-c
	               Show(4) // Show control
	            )
	            (case $1900 // alt-p
	               Show(2) // Show priority
	            )
	            (case $3200 // alt-m
	               // Show memory usage
	                  ShowFree()
	                  FormatPrint(
	                      "Free Heap: %u Bytes\nLargest ptr: %u Bytes\nFreeHunk: %u KBytes\nLargest hunk: %u Bytes"
	                      MemoryInfo(miFREEHEAP)
	                      MemoryInfo(miLARGESTPTR)
	                      (>> MemoryInfo(miFREEHUNK) 6)
	                      MemoryInfo(miLARGESTHUNK)
	                  )
	            )
	            (case $1400 // alt-t
	               // teleport to room
	                    (send gRoom:newRoom(GetNumber("Which Room Number?")))
	            )
	            (case $1700 // alt-i
	               // get inventory
	               (send gEgo:get(GetNumber("Which inventory#?")))
	            )
	            (case $1f00 // alt-s
	               // Show cast
	               (send gCast:eachElementDo(#showSelf))
	            )
	         )
	      )
	      ////////////////////////////////////////////////////
	      // End of debug functionality                     //
	      ////////////////////////////////////////////////////
		
		(super:handleEvent(pEvent))
		(if((<> (send pEvent:type) evSAID) or (send pEvent:claimed))
		    return(TRUE)
		)

        /***********************************/
        /* Add global said statements here */
        /***********************************/
        
   /*     (if(Said('hi'))
            Print("Well hello to you too!")
        )
        */
        (if(Said('remove>'))
        	(if (Said('/cross'))
				(if (send gEgo:has(INV_CROSS))
					(if(== gWearingCross TRUE)
						Print("You remove the silver cross from your neck.")
						= gWearingCross FALSE
					)(else
						Print("You're not wearing the cross.")					 
				 	)
				)(else
					PrintDontHaveIt()
			 	)
			)
			(if (Said('/cloak'))
				(if (send gEgo:has(INV_CLOAK))
					(switch(gWearingCloakRing)
						(case 0
							Print("You're not wearing the cloak.")
						)
						(case 1
							Print("You take off the cloak.")
							= gWearingCloakRing 0
						)
						(case 2
							Print("You're not wearing the cloak.")
						)
						(case 3
							Print("You take off the cloak.")
							= gWearingCloakRing 2
						)
					)
				)(else
					PrintDontHaveIt()
			 	)
			)
			(if (Said('/ring'))
				(if (send gEgo:has(INV_RING))
					(switch(gWearingCloakRing)
						(case 0
							Print("You're not wearing the ring.")
						)
						(case 1
							Print("You're not wearing the ring.")
						)
						(case 2
							Print("You take off the ring.")
							= gWearingCloakRing 0
						)
						(case 3
							Print("You take off the ring.")
							= gWearingCloakRing 1
							
						)
					)
				)(else
					PrintDontHaveIt()
			 	)
			)
		)
		(if(Said('take>'))			
			(if (Said('/cloth'))
				(if (send gEgo:has(INV_BOTTLE))
					Print("You pull a large piece of cloth out of the bottle.")
					(send gEgo:put(INV_BOTTLE)get(INV_EMPTYBOTTLE)get(INV_CLOTH))
					(send gGame:changeScore(2))
				)(else
					Print("You don't have the cloth.")
				 )
			)
			(if (Said('<off>'))
				(if (Said('/cross'))
					(if (send gEgo:has(INV_CROSS))
						(if(== gWearingCross TRUE)
							Print("You remove the silver cross from your neck.")
							= gWearingCross FALSE
						)(else
							Print("You're not wearing the cross.")					 
					 	)
					)(else
						PrintDontHaveIt()
				 	)
				)
				(if (Said('/cloak'))
					(if (send gEgo:has(INV_CLOAK))
						(switch(gWearingCloakRing)
							(case 0
								Print("You're not wearing the cloak.")
							)
							(case 1
								Print("You take off the cloak.")
								= gWearingCloakRing 0
							)
							(case 2
								Print("You're not wearing the cloak.")
							)
							(case 3
								Print("You take off the cloak.")
								= gWearingCloakRing 2
							)
						)
					)(else
						PrintDontHaveIt()
				 	)
				)
				(if (Said('/ring'))
					(if (send gEgo:has(INV_RING))
						(switch(gWearingCloakRing)
							(case 0
								Print("You're not wearing the ring.")
							)
							(case 1
								Print("You're not wearing the ring.")
							)
							(case 2
								Print("You take off the ring.")
								= gWearingCloakRing 0
							)
							(case 3
								Print("You take off the ring.")
								= gWearingCloakRing 1
								
							)
						)
					)(else
						PrintDontHaveIt()
				 	)
				)
			)
			(if(Said('[/*]'))
				Print("You can't take that.")
			)
		)
        (if(Said('talk/*'))
            Print("You say a few words to it, but get no response.")
        )
        (if(Said('eat>'))
        	(if(Said('/goodies'))
				(if (send gEgo:has(INV_BASKET))
					(send gEgo:put(INV_BASKET))
					(send gEgo:get(INV_EMPTYBASKET))
					(send gGame:changeScore(-2))
					Print("Yum, yum. The snacks from the goody basket are sure tasty.")
				)(else
					(if (send gEgo:has(INV_EMPTYBASKET))
						Print("The basket of goodies is empty because you ate all of it.")
					)(else
						Print("You are not carrying it.")
					 )
				 )
			)
        	(if(Said('/*'))
        		Print("That doesn't appeal just now.")
			)
		)
		(if(Said('rub>'))
			(if(Said('/lamp'))
				(if (send gEgo:has(INV_LAMP))
					(self:setScript(GenieScript))
				)(else
					Print("You aren't carrying it.")
				 )
			)
		)
		(if(Said('ride>'))
			(if (Said('/carpet'))
				(if (send gEgo:has(INV_CARPET))
					(if(== gEgoIsSwimming FALSE)
						(if (== gCanFlyCarpet TRUE)
							(self:setScript(CarpetScript))
						)(else
							Print("You can't ride the carpet just now.")
					 	)
					)(else
						Print("You'll have to get out of the water first.")
					 )
				)(else
					PrintDontHaveIt()
				 )
			)
		)
		(if(Said('die'))
			(send gEgo:hide())
			= dyingScript ScriptID(DYING_SCRIPT)
 			(send dyingScript:
    			caller(699)
    			register("You have typed the word 'Die'. So you did.")
  			)
  			(send gGame:setScript(dyingScript))
		)
		(if(Said('look<in/basket'))
			(if (send gEgo:has(INV_BASKET))
				Print("You look in the basket of goodies and see many good things to eat: cookies, fruit, crackers, and sandwiches.")
			)(else
				(if (send gEgo:has(INV_EMPTYBASKET))
					Print("The basket of goodies is empty because you ate all of it.")
				)(else
					Print("You don't have a basket.")
				 )
			 )
		)
		(if (Said('open>'))
			(if (Said('/cage,birdcage'))
				(if (send gEgo:has(INV_NIGHTINGALE))
					Print("You open the pretty birdcage and the nightingale swoops out. The bird is gone. You have nothing but an empty birdcage.")
					(send gEgo:put(INV_NIGHTINGALE)get(INV_EMPTYCAGE))
					(send gGame:changeScore(-2))
				)(else
					PrintCantDoThat()
				 )
			)
		)
		(if (Said('free>'))
			(if (Said('/bird,nightingale'))
				(if (send gEgo:has(INV_NIGHTINGALE))
					Print("You open the pretty birdcage and the nightingale swoops out. The bird is gone. You have nothing but an empty birdcage.")
					(send gEgo:put(INV_NIGHTINGALE)get(INV_EMPTYCAGE))
					(send gGame:changeScore(-2))
				)(else
					PrintCantDoThat()
				 )
			)
		)		
		(if (Said('wear>'))
			(if (Said('/cross'))
				(if (send gEgo:has(INV_CROSS))
					(if(== gWearingCross FALSE)
						Print("You put the silver cross necklace around your neck.")
						= gWearingCross TRUE
					)(else
						Print("You're already wearing the cross.")					 
					 )
				)(else
					PrintDontHaveIt()
				 )
			)
			(if (Said('/cloak'))
				(if (send gEgo:has(INV_CLOAK))
					(switch(gWearingCloakRing)
						(case 0
							Print("You wear the cloak.")
							= gWearingCloakRing 1
						)
						(case 1
							Print("You're already wearing the cloak.")
						)
						(case 2
							Print("You wear the cloak.")
							= gWearingCloakRing 3
						)
						(case 3
							Print("You're already wearing the cloak.")
						)
					)
				)(else
					PrintDontHaveIt()
				 )
			)
			(if (Said('/ring'))
				(if (send gEgo:has(INV_RING))
					(switch(gWearingCloakRing)
						(case 0
							Print("You slip the large ruby ring on your finger.")
							= gWearingCloakRing 2
						)
						(case 1
							Print("You slip the large ruby ring on your finger.")	
							= gWearingCloakRing 3						
						)
						(case 2
							Print("You're already wearing the ring.")
						)
						(case 3
							Print("You're already wearing the ring.")
						)
					)
				)(else
					PrintDontHaveIt()
				 )
			)
			(if (Said('/*'))
				Print("You can't wear that.")
			)
			(if (Said('[/!*]'))
				Print("What do you want to wear?")
			)
		)
		return(FALSE)
  	)
)


/******************************************************************************/
(class Iitem of InvI
	(properties
		said 0
		description 0
		owner 0
		view 0
		loop 0
		cel 0
		script 0
	)
	(method (showSelf)
		Print(
            description
			#title objectName
			#icon view 0 0
		)
	)
)
/******************************************************************************/
(instance statusCode  of Code
	(properties)
 	(method (doit param1)
//		Format(param1 " Score: %d of %-3d             %02d:%02d:%02d               " gScore gMaxScore gTimeHours gTimeMinutes gTimeSeconds)
 		Format(param1 " Score: %d of %-3d             King's Quest II        " gScore gMaxScore)
 	)
)
/******************************************************************************/
(instance ego of Ego
	(properties
		y 1111
		x 0
		z 0
		heading 0
		yStep 2
		view 0
		loop 0
		cel 0
		priority 0
		underBits 0
		signal $2000
		nsTop 0
		nsLeft 0
		nsBottom 0
		nsRight 0
		lsTop 0
		lsLeft 0
		lsBottom 0
		lsRight 0
		brTop 0
		brLeft 0
		brBottom 0
		brRight 0
		cycleSpeed 0
		script 0
		cycler 0
		timer 0
		illegalBits $8000
		xLast 0
		yLast 0
		xStep 3
		moveSpeed 0
		blocks 0
		baseSetter 0
		mover 0
		looper 0
		viewer 0
		avoider 0
		edgeHit 0
	)
)
/******************************************************************************/
(instance scoreSound of Sound
	(properties
		state 0
		number SCORE_SOUND
		priority 10
		loop 1
		handle 0
		signal 0
		prevSignal 0
		client 0
		owner 0
	)
)
/******************************************************************************/
(instance theMusic  of Sound
	(properties
		state 0
		number 1
		priority 0
		loop 1
		handle 0
		signal 0
		prevSignal 0
		client 0
		owner 0
	)
)
/******************************************************************************/
(instance theSoundFX  of Sound
	(properties
		state 0
		number 1
		priority 5
		loop 1
		handle 0
		signal 0
		prevSignal 0
		client 0
		owner 0
	)
)
/******************************************************************************/
(instance theWindow  of SysWindow
	(properties)
	(method (open)
		(if(< Graph(grGET_COLOURS) 9)
			(if( (< color 7) or (== color 8) )
				= color 0
				= back  15
			)(else
				= color 15
				= back  0
			)
  		)
  		(super:open())
	)
)
/******************************************************************************/
(instance NormalBase of Code
	(properties)
	(method (doit)
		(var temp0)
		(if(== gRoomNumberExit 253)
		    = temp0 22
		)(else
  			= temp0 10
		)
		(send gEgo:
			brBottom(
				(+ (send gEgo:y) 1)
			)
		)
		(send gEgo:
			brTop(
				(- (send gEgo:brBottom) (send gEgo:yStep))
			)
		)
		(send gEgo:
			brLeft(
				(- (send gEgo:x) temp0)
			)
		)
		(send gEgo:
			brRight(
				(+ (send gEgo:x) temp0)
			)
		)
	)
)

/******************************************************************************
 * THE PUBLIC PROCEDURES
 ******************************************************************************/
(procedure public (SetUpEgo theLoop theView)
	PlayerControl()
	(send gEgo:edgeHit(EDGE_NONE))
	(switch(paramTotal)
		(case 0
			SetUpActor(gEgo (send gEgo:loop) gEgoView)
		)
  		(case 1
			SetUpActor(gEgo theLoop gEgoView)
		)
  		(case 2
		    SetUpActor(gEgo theLoop theView)
  		)
	)
)
/******************************************************************************/
(procedure public (SetUpActor pActor theLoop theView)
	(if(> paramTotal 1)
		(send pActor:loop(theLoop))
	)
	(if(> paramTotal 2)
		(send pActor:view(theView))
	)
	(send pActor:
		setLoop(-1)
		setPri(-1)
		setStep(3 2)
        setCycle(StopWalk gEgoView gEgoStoppedView)
		illegalBits($8000)
		cycleSpeed(0)
		moveSpeed(0)
		ignoreActors(0)
	)
)
/******************************************************************************/
(procedure public (ProgramControl)
	(User:
		canControl(FALSE)
		canInput(FALSE)
	)
	(send gEgo:setMotion(NULL))
)
/******************************************************************************/
(procedure public (PlayerControl)
	(User:
		canControl(TRUE)
		canInput(TRUE)
	)
	(send gEgo:setMotion(NULL))
)
/******************************************************************************/
(procedure public (DisposePrintDlg)
	(if(gPrintDlg)
		(send gPrintDlg:dispose())
	)
)
/******************************************************************************/
(procedure public (PrintOK)
	Print("O.K.")
)
/******************************************************************************/
(procedure public (PrintItIs)
	Print("It is.")
)
/******************************************************************************/
(procedure public (PrintYouAre)
	Print("You are.")
)
/******************************************************************************/
(procedure public (PrintGoodIdea)
	Print("Good idea. You might try that again later.")
)
/******************************************************************************/
(procedure public (PrintNotCloseEnough)
	Print("You're not close enough.")
)
/******************************************************************************/
(procedure public (PrintAlreadyTookIt)
	Print("You've already taken it.")
)
/******************************************************************************/
(procedure public (PrintDontHaveIt)
	Print("You don't have it.")
)
/******************************************************************************/
(procedure public (PrintGetCloser)
	Print("Get closer.")
)
/******************************************************************************/
(procedure public (PrintDontSeeIt)
	Print("You don't see that here.")
)
/******************************************************************************/
(procedure public (PrintCantDoThat mem)
	(if(> MemoryInfo(miLARGESTPTR) mem)
		return(TRUE)
	)(else
		Print("You can't do that here; at least, not now.")
		return(FALSE)
	)
)
/******************************************************************************/
(procedure public (AddViewToPic pView)
	(var hView)
	(if(pView)
		= hView (View:new())
		(send hView:
		  	view(send pView:view)
			loop(send pView:loop)
			cel(send pView:cel)
			priority(send pView:priority)
			posn( (send pView:x) (send pView:y) )
			addToPic()
		)
		(send pView:
			posn( (send pView:x) (+ 1000 (send pView:y)) )
		)
	)
)
/******************************************************************************/
(procedure public (SetInvIOwner index owner)
	(var hInvI)
	= hInvI (send gInv:at(index))
	(if(< paramTotal 2)
		(send hInvI:owner(gRoomNumberExit))
	)(else
		(send hInvI:owner(owner))
	)
)
(procedure public (StartCutscene)
	= gProgramControl TRUE
	ProgramControl()
	(SL:disable())
	(TheMenuBar:hide()state(DISABLED))
)
(procedure public (EndCutscene)
	= gProgramControl FALSE
	PlayerControl()
	(TheMenuBar:draw()state(ENABLED))
	(SL:enable())
	SetCursor(999 HaveMouse())	
)

/******************************************************************************
 * THE INVENTORY ITEMS                                                        *
 ******************************************************************************/
(instance Nothing  of Iitem
	(properties)
)
/******************************************************************************/
(instance {Sword} of Iitem
	(properties
		said 'sword'
		description
             "The shining sword is not incredibly large, but is very sharp! It has a snake carved into the bronze handle."
		owner 0
		view 800
		loop 0
		cel 0
		script 0
	)
)
(instance {Trident} of Iitem
	(properties
		said 'trident'
		description
             "The trident is rusty and corroded. It looks somewhat like a pitchfork."
		owner 0
		view 801
		loop 0
		cel 0
		script 0
	)
)
(instance {Bouquet} of Iitem
	(properties
		said 'flowers [ < bouquet < of ]'
		description
             "The bouquet of wild flowers is very pretty. The sweet scent is delightful."
		owner 0
		view 802
		loop 0
		cel 0
		script 0
	)
)
(instance {Bracelet} of Iitem
	(properties
		said 'bracelet'
		description
             "The bracelet looks like it would fit a small wrist. It is a circlet of diamonds surrounded by beautiful blue sapphires."
		owner 0
		view 803
		loop 0
		cel 0
		script 0
	)
)
(instance {Stake} of Iitem
	(properties
		said 'stake [ < wooden ]'
		description
             "It appears to be an ordinary tent stake."
		owner 0
		view 804
		loop 0
		cel 0
		script 0
	)
)
(instance {Ruby Ring} of Iitem
	(properties
		said 'ring [ < ruby ]'
		description
             "The ring is gold with an enormous red ruby. The initials 'C.D' are engraved inside the gold band."
		owner 0
		view 805
		loop 0
		cel 0
		script 0
	)
)
(instance {Tiara} of Iitem
	(properties
		said 'tiara'
		description
             "This tiara is stunning! It is encrusted with diamonds from the back to the front, progressively getting larger. The center stone in the front is a dazzling sapphire."
		owner 0
		view 806
		loop 0
		cel 0
		script 0
	)
)
(instance {Necklace} of Iitem
	(properties
		said 'necklace'
		description
             "This is a dazzling sapphire necklace."
		owner 0
		view 807
		loop 0
		cel 0
		script 0
	)
)
(instance {Earrings} of Iitem
	(properties
		said 'earrings'
		description
             "The diamond and sapphire earrings are very delicate. Each earring has a large sapphire surrounded by glittering diamonds."
		owner 0
		view 808
		loop 0
		cel 0
		script 0
	)
)
(instance {Brooch} of Iitem
	(properties
		said 'brooch'
		description
             "The brooch has an enormous sapphire encircled by two rows of glittering diamonds!"
		owner 0
		view 809
		loop 0
		cel 0
		script 0
	)
)
(instance {Mallet} of Iitem
	(properties
		said 'mallet'
		description
             "The head of the mallet is made of iron. It has a long wooden handle."
		owner 0
		view 810
		loop 0
		cel 0
		script 0
	)
)
(instance {Gold Key} of Iitem
	(properties
		said 'key [ < gold ]'
		description
             "It is a shiny gold key."
		owner 0
		view 811
		loop 0
		cel 0
		script 0
	)
)
(instance {Empty Basket} of Iitem
	(properties
		said 'basket [ < empty ] , goodies < basket < of'
		description
             "The basket of goodies is empty because you ate all of it."
		owner 0
		view 812
		loop 0
		cel 0
		script 0
	)
)
(instance {Basket} of Iitem
	(properties
		said 'basket , goodies [ < basket < of ]'
		description
             "There are many good things to eat in the basket of goodies."
		owner 0
		view 813
		loop 0
		cel 0
		script 0
	)
)
(instance {Bottle} of Iitem
	(properties
		said 'bottle'
		description
             "There is a large cloth in the glass bottle."
		owner 0
		view 814
		loop 0
		cel 0
		script 0
	)
)
(instance {Empty Bottle} of Iitem
	(properties
		said 'bottle [ < empty ]'
		description
             "The glass bottle is empty."
		owner 0
		view 815
		loop 0
		cel 0
		script 0
	)
)
(instance {Ham} of Iitem
	(properties
		said 'ham'
		description
             "The smoked ham has a wonderful, savory smell. It's hard to resist tasting it. It looks as though someone has taken a bite out of it."
		owner 0
		view 816
		loop 0
		cel 0
		script 0
	)
)
(instance {Chicken Soup} of Iitem
	(properties
		said 'soup [ < chicken ]'
		description
             "The pot contains hot, bubbling chicken soup."
		owner 0
		view 817
		loop 0
		cel 0
		script 0
	)
)
(instance {Black Cloak} of Iitem
	(properties
		said 'cloak [ < black ]'
		description
             "The black cloak would better fit a man taller than you. Red satin lines the inside."
		owner 0
		view 818
		loop 0
		cel 0
		script 0
	)
)
(instance {Cross} of Iitem
	(properties
		said 'cross'
		description
             "You see a shiny silver cross dangling from a silver chain."
		owner 0
		view 819
		loop 0
		cel 0
		script 0
	)
)
(instance {Caged Nightingale} of Iitem
	(properties
		said 'nightingale [ < caged ] , cage '
		description
             "As you look at the bird it begins to sing merrily."
		owner 0
		view 820
		loop 0
		cel 0
		script 0
	)
)
(instance {Empty Birdcage} of Iitem
	(properties
		said 'cage [ < empty ]'
		description
             "The gilded bird cage is empty. The nightingale has flown away."
		owner 0
		view 821
		loop 0
		cel 0
		script 0
	)
)
(instance {Oil Lamp} of Iitem
	(properties
		said 'lamp [ < oil ] '
		description
             "The old oil lamp is fashioned of brass that has been tarnished. There is a spout at one end, and a round handle at the other. The lamp is empty inside."
		owner 0
		view 822
		loop 0
		cel 0
		script 0
	)
)
(instance {Cloth} of Iitem
	(properties
		said 'cloth'
		description
             "It is a large sheet of cloth."
		owner 0
		view 823
		loop 0
		cel 0
		script 0
	)
)
(instance {Pillow} of Iitem
	(properties
		said 'pillow'
		description
             "The pillow is small, and made of shiny red satin."
		owner 0
		view 824
		loop 0
		cel 0
		script 0
	)
)
(instance {Fishing Net} of Iitem
	(properties
		said ' net [ < fishing ] '
		description
             "The fishing net is fashioned from a material that is unfamiliar to you. It is smooth and tangle-proof."
		owner 0
		view 825
		loop 0
		cel 0
		script 0
	)
)
(instance {Carpet} of Iitem
	(properties
		said ' carpet [ < magic ] '
		description
             "The magic carpet is a beautifully woven persian rug with a colorful fringe on the ends."
		owner 0
		view 826
		loop 0
		cel 0
		script 0
	)
)
(instance {Bridle} of Iitem
	(properties
		said 'bridle'
		description
             "The leather bridle is studded with silver rivets and a silver bit."
		owner 0
		view 827
		loop 0
		cel 0
		script 0
	)
)
(instance {Candle} of Iitem
	(properties
		said 'candle'
		description
             "The wax candle burns brightly. Ouch! Hot wax dripped on your finger."
		owner 0
		view 828
		loop 0
		cel 0
		script 0
	)
)
(instance {Sugar Cube} of Iitem
	(properties
		said ' cube [ < sugar ] , sugar '
		description
             "It looks like a normal sugar cube. It does have a bit of a sparkle, though."
		owner 0
		view 829
		loop 0
		cel 0
		script 0
	)
)
(instance {Gasping Fish} of Iitem
	(properties
		said ' fish [ < gasping ] '
		description
             "The large fish is covered with shimmering golden scales. Its graceful fins and tail are almost translucent. Right now, the beautiful fish is writhing in agony."
		owner 0
		view 830
		loop 0
		cel 0
		script 0
	)
)
(instance {Amulet} of Iitem
	(properties
		said 'amulet'
		description
             "You examine the amulet. It is made of bronze that has tarnished with age. A Tiger's Eye stone is imbedded into the front. You turn the amulet over, and see the word `HOME' engraved on the back."
		owner 0
		view 831
		loop 0
		cel 0
		script 0
	)
)
(instance {Clamshell} of Iitem
	(properties
		said '(clamshell , shell [ < clam ], clam)'
		description
             "This is just an ordinary clamshell. There doesn't seem to be anything special about it."
		owner 0
		view 832
		loop 0
		cel 0
		script 0
	)
)
(instance {Silver Key} of Iitem
	(properties
		said 'key [ < silver ]'
		description
             "It is a shiny silver key."
		owner 0
		view 833
		loop 0
		cel 0
		script 0
	)
)
/******************************************************************************/
(instance GenieScript of Script
	(properties)
	(method (changeState newState)
		(= state newState)
		(switch (newState)
			(case 0 = cycles 40
				ProgramControl()
				= gProgramControl TRUE
				(send gEgo:hide())
				(Genie:init()posn((send gEgo:x) (send gEgo:y))setCycle(Fwd))
			)
			(case 1
				(switch(gRubLamp)
					(case 0
						Print("When you rub the tarnished brass lamp, you hear a POOOOFF and a genie appears! \"Master,\" he says, \"I leave a gift for you. A magic carpet!\" The genie then poofs back into the lamp.")
						++gRubLamp
						(send gEgo:get(INV_CARPET))
						(send gGame:changeScore(2))
					)
					(case 1
						Print("When you rub the lamp, you hear a loud POOOOFF and a genie magically appears. \"Master,\" he says, \"I leave a gift for you. A beautiful sword!\" The genie then disappears in a puff of smoke.")
						++gRubLamp
						(send gEgo:get(INV_SWORD))
						(send gGame:changeScore(2))
					)
					(case 2
						Print("Using your sleeve as a cloth you rub the lamp, causing the genie to appear again. \"Master,\" he says, \"I leave a gift for you. A leather bridle!\" The genie then pops back into the lamp.")
						Print("Oh, no! The lamp has disappeared from your hands!")
						(send gEgo:get(INV_BRIDLE)put(INV_LAMP))
						(send gGame:changeScore(2))
					)
				)
				(self:cue())
			)
			(case 2
				(Genie:dispose())
				(send gEgo:show()loop(0))
				PlayerControl()
				= gProgramControl FALSE
			)
		)
	)
)
(instance Genie of Prop
	(properties
		x 0
		y 0
		view 55
		cycleSpeed 1
	)
)
(instance EgoCarpet of Act
	(properties
		x 0
		y 0
		view 15		
	)
)
(instance CarpetScript of Script
	(properties)
	(method (changeState newState)
		= state newState
		(switch (newState)
			(case 0
				ProgramControl()
				= gProgramControl TRUE
				= gCarpetFlying TRUE
				(send gEgo:hide())
				(send gGame:setSpeed(3))
				(EgoCarpet:
					setPri(send gEgo:priority())
					setStep(4 4)
					ignoreControl(ctlWHITE)
					init()
					posn((send gEgo:x) (send gEgo:y))
					loop(send gEgo:loop())
					setCycle(Fwd)
					setMotion(MoveTo (+ (send gEgo:x) 5) 0 CarpetScript)
				)
			)
			(case 1
				(send gRoom:newRoom(105))
			)
		)
	)
)
