/******************************************************************************
 King's Quest II SCI Remake
 By Brandon Blume
 ******************************************************************************
 window.sc
 Contains the extension of the SysWindow class.
 ******************************************************************************/
(include "sci.sh")
(include "game.sh")
/******************************************************************************/
(script WINDOW_SCRIPT)
/******************************************************************************/
(use "syswindow")
(use "main")
(local
	topPort
)
/******************************************************************************/
(class Window of SysWindow
	(properties
		top 0
		left 0
		bottom 0
		right 0
		color clBLACK
		back clWHITE
		priority -1
		window 0
		type 0
		title 0
		brTop 0
		brLeft 0
		brBottom 190
		brRight 320
		underBits 0
	)
	(method (doit))
	(method (dispose)
		(self:restore())
		(if(window)
			DisposeWindow(window)
			= window 0
		)
		(super:dispose())
	)
	(method (open)
		= window NewWindow(
			top left bottom right
		    title
		    type
		    priority
		    color
		    back
		)
	)
	(method (handleEvent)
		return(FALSE)
	)
	(method (setMapSet)
		(var mapSet)
		= mapSet 0
		(if(<> -1 color)
		    = mapSet (| mapSet 1)
		)
		(if(<> -1 priority)
		    = mapSet (| mapSet 2)
		)
		return(mapSet)
	)
	(method (move theX theY)
		= left   (+ left   theX)
		= right  (+ right  theX)
		= top  (+ top  theY)
		= bottom (+ bottom theY)
	)
	(method (moveTo newX newY)
		(self:move( (- newX left) (- newY top) ))
	)
	(method (draw newColor newPriority)
		(if(>= paramTotal 1)
			= color newColor
		)
		(if(>= paramTotal 2)
			= priority newPriority
		)
		Graph(
			11
			top left right bottom
			(self:setMapSet())
			color
			priority
		)
	)
	(method (save)
		= underBits Graph(
			grSAVE_BOX
			top left bottom right
			(self:setMapSet())
		)
	)
	(method (restore)
		(if(underBits)
			Graph(grRESTORE_BOX underBits)
		)
	)
	(method (inset theX theY)
		= top (+ top theY)
		= left (+ left theX)
		= bottom (- bottom theY)
		= right (- right theX)
	)
	(method (show)
		Graph(
			grUPDATE_BOX
			top left bottom right
			(self:setMapSet())
		)
	)
	(method (erase)
		(self:draw(back -1))
	)
	(method (center)
		(self:moveTo(
			(/ (- (- brRight left) (- right left)) 2)
		    (/ (- (- brBottom top) (- bottom top)) 2)
		))
	)
)
/******************************************************************************/
(class TextWindow of Window
	(properties
		top 0
		left 0
		bottom 0
		right 0
		color clBLACK
		back clWHITE
		type nwNOFRAME
		title 0
		centertext FALSE
	)
	(method (drawtitle)
		= type nwTITLE
	)
	(method (drawborder)
		= type nwNORMAL
	)
	(method (drawgui)
		= type nwNOFRAME
	)
	(method (setx X)
		= left X
	)
	(method (sety Y)
		= top Y
	)
	(method (setdim theW theH)
		= right (+ left theW)
		= bottom (+ top theH)
	)
	(method (movex theX)
		= left (+ left theX)
		= right (+ right theX)
	)
	(method (movey theY)
		= top (+ top theY)
		= bottom (+ bottom theY)
	)
	(method (movexto newX)
		(self:movex(- newX left))
	)
	(method (moveyto newY)
		(self:movey(- newY top))
	)
	(method (hcenter theY)
		(self:movexto(/ (- brRight (- right left)) 2))
		(self:moveyto(theY))
	)
	(method (vcenter theX)
		(self:moveyto(/ (- brBottom (- bottom top)) 2))
		(self:movexto(theX))
	)
	(method (displaytext text line)
		(var w)
		= w (- right left)

		SetPort(window)
		(if (centertext)
			Display(text line
				dsCOORD 4 4
				dsWIDTH (- w 8)
				dsFONT 4
				dsALIGN alCENTER
				)
		)(else
			Display(text line
				dsCOORD 4 4
				dsWIDTH (- w 8)
				dsFONT 4
				   )
		 )
		SetPort(gOldPort)
	)
	(method (displaygui text line)
		(var w, h, w2, h2, x, y, hCenter, vCenter, guiWidth, guiHeight)
		= guiWidth (CelWide(gGuiView gGuiLoop gGuiTopLeft))
		= guiHeight (CelHigh(gGuiView gGuiLoop gGuiTopLeft))
		= w (- right left)
		= h (- bottom top)
		= w2 (- w guiWidth)
		= h2 (- h guiHeight)
		
		SetPort(window)

		(for(= x guiWidth) (< x w2) (= x + x guiWidth)
			DrawCel(gGuiView gGuiLoop gGuiTop    x  0 nwON_TOP)			//top edge cel
			DrawCel(gGuiView gGuiLoop gGuiBottom x h2 nwON_TOP)			//bottom edge cel
		)
		(for(= y guiHeight) (< y h2) (= y + y guiHeight)
			DrawCel(gGuiView gGuiLoop gGuiLeft    0 y nwON_TOP)			//left edge cel
			DrawCel(gGuiView gGuiLoop gGuiRight  w2 y nwON_TOP)			//right edge cel
		)
		DrawCel(gGuiView gGuiLoop gGuiTopLeft      0  0 nwON_TOP)  		//topleft corner cel
		DrawCel(gGuiView gGuiLoop gGuiTopRight    w2  0 nwON_TOP)  		//topright corner cel
		DrawCel(gGuiView gGuiLoop gGuiBottomLeft   0 h2 nwON_TOP)  		//bottomleft corner cel
		DrawCel(gGuiView gGuiLoop gGuiBottomRight w2 h2 nwON_TOP)  		//bottomright corner cel
				
		(if (centertext)
			Display(text line
				dsCOORD 8 8
				dsWIDTH (- w 18)
				dsFONT 4
				dsALIGN alCENTER
				)
		)(else
			Display(text line
				dsCOORD 8 8
				dsWIDTH (- w 18)
				dsFONT 4
				   )
		 )
		SetPort(gOldPort)
//		DrawCel(899 1 0 (- (+ left (/ w 2)) 27) (- top 10) nwON_TOP)
	)
	(method (drawCrown)
		(var w, h)
		= w (- right left)
		= h (- bottom top) 
		= topPort NewWindow(
			(- top 10) (- (+ left (/ w 2)) 27) top (+ (- right (/ w 2)) 28)
			""
			nwTRANSPARENT_NOFRAME
			nwON_TOP
			0
			0
					   	)
		SetPort(topPort)					   
    	DrawCel(899 1 0 0 0 nwON_TOP)
	)
)